@@ -28,7 +28,7 @@ from email import charset
 charset.add_charset('utf-8', None)
 emlpolicy = email.policy.EmailPolicy(utf8=True, cte_type='8bit', max_line_length=None)
 
-__VERSION__ = '0.5.0'
+__VERSION__ = '0.5.1'
 ATTESTATION_FORMAT_VER = '0.1'
 
 logger = logging.getLogger('b4')
@@ -38,8 +38,8 @@ FILENAME_RE = re.compile(r'^(---|\+\+\+) (\S+)')
 
 PASS_SIMPLE = '[P]'
 FAIL_SIMPLE = '[F]'
-PASS_FANCY = '[\033[32m✓\033[0m]'
-FAIL_FANCY = '[\033[31m✗\033[0m]'
+PASS_FANCY = '[\033[32m\u2713\033[0m]'
+FAIL_FANCY = '[\033[31m\u2717\034[0m]'
 
 # You can use bash-style globbing here
 WANTHDRS = [
@@ -200,7 +200,7 @@ class LoreMailbox:
                 if reused:
                     continue
                 # Try to backfill from that project
-                tmp_mbox = mkstemp()[1]
+                tmp_mbox = mkstemp('b4-backfill-mbox')[1]
                 get_pi_thread_by_msgid(patch.msgid, tmp_mbox, useproject=projmap[entry[1]])
                 mbx = mailbox.mbox(tmp_mbox)
                 was = len(self.msgid_map)
@@ -284,7 +284,7 @@ class LoreMailbox:
 
             trailers, mismatches = fmsg.get_trailers(sloppy=sloppytrailers)
             for tname, tvalue in mismatches:
-                lser.trailer_mismatches.add((tvalue, fmsg.fromname, fmsg.fromemail))
+                lser.trailer_mismatches.add((tname, tvalue, fmsg.fromname, fmsg.fromemail))
             lvl = 1
             while True:
                 logger.debug('%sParent: %s', ' ' * lvl, pmsg.full_subject)
@@ -775,6 +775,7 @@ class LoreMessage:
 
         # Body and body-based info
         self.body = None
+        self.charset = 'utf-8'
         self.has_diff = False
         self.has_diffstat = False
         self.trailers = set()
@@ -830,6 +831,7 @@ class LoreMessage:
         mcharset = self.msg.get_content_charset()
         if not mcharset:
             mcharset = 'utf-8'
+        self.charset = mcharset
 
         for part in msg.walk():
             cte = part.get_content_type()
@@ -843,6 +845,7 @@ class LoreMessage:
                 pcharset = mcharset
             try:
                 payload = payload.decode(pcharset, errors='replace')
+                self.charset = pcharset
             except LookupError:
                 # what kind of encoding is that?
                 # Whatever, we'll use utf-8 and hope for the best
@@ -878,7 +881,7 @@ class LoreMessage:
             # Do we have something that looks like a person-trailer?
             matches = re.findall(r'^\s*([\w-]+):[ \t]+(.*<\S+>)\s*$', self.body, re.MULTILINE)
             # These are commonly part of patch/commit metadata
-            badtrailers = ('from', 'author')
+            badtrailers = ('from', 'author', 'cc')
             if matches:
                 for tname, tvalue in matches:
                     if tname.lower() not in badtrailers:
@@ -1131,6 +1134,7 @@ class LoreMessage:
     def load_hashes(self):
         if self.attestation is not None:
             return
+        logger.debug('Calculating hashes for: %s', self.full_subject)
         msg_out = mkstemp()
         patch_out = mkstemp()
         cmdargs = ['mailinfo', '--encoding=UTF-8', msg_out[1], patch_out[1]]
@@ -1157,8 +1161,8 @@ class LoreMessage:
         os.unlink(msg_out[1])
 
         p = None
-        with open(patch_out[1], 'r') as pfh:
-            patch = pfh.read()
+        with open(patch_out[1], 'rb') as pfh:
+            patch = pfh.read().decode(self.charset, errors='replace')
             if len(patch.strip()):
                 diff = LoreMessage.get_clean_diff(patch)
                 phasher = hashlib.sha256()
@@ -1195,6 +1199,11 @@ class LoreMessage:
         parts = body.split('\n---\n', 1)
         if len(parts) == 2:
             basement = parts[1].rstrip('\n')
+        elif body.find('\ndiff ') >= 0:
+            parts = body.split('\ndiff ', 1)
+            if len(parts) == 2:
+                parts[1] = 'diff ' + parts[1]
+            basement = parts[1].rstrip('\n')
 
         mbody = parts[0].strip('\n')
 
@@ -1919,8 +1928,12 @@ def save_strict_thread(in_mbx, out_mbx, msgid):
             logger.debug('Looking at: %s', c_msgid)
 
             refs = set()
-            for ref in msg.get('References', msg.get('In-Reply-To', '')).split():
-                ref = ref.strip().strip('<>')
+            msgrefs = list()
+            if msg.get('In-Reply-To', None):
+                msgrefs += email.utils.getaddresses([str(x) for x in msg.get_all('in-reply-to', [])])
+            if msg.get('References', None):
+                msgrefs += email.utils.getaddresses([str(x) for x in msg.get_all('references', [])])
+            for ref in set([x[1] for x in msgrefs]):
                 if ref in got or ref in want:
                     want.add(c_msgid)
                 elif len(ref):
@@ -1941,6 +1954,11 @@ def save_strict_thread(in_mbx, out_mbx, msgid):
                     # Add all these to want
                     want.update(maybe[c_msgid])
                     maybe.pop(c_msgid)
+                # Add all maybes that have the same ref into want
+                for ref in refs:
+                    if ref in maybe:
+                        want.update(maybe[ref])
+                        maybe.pop(ref)
 
         # Remove any entries not in "seen" (missing messages)
         for c_msgid in set(want):
@@ -2003,7 +2021,9 @@ def get_pi_thread_by_msgid(msgid, savefile, useproject=None, nocache=False):
     logger.debug('t_mbx_url=%s', t_mbx_url)
 
     logger.critical('Grabbing thread from %s', projurl.split('://')[1])
-    in_mbxf = get_pi_thread_by_url(t_mbx_url, '%s-loose' % savefile, nocache=nocache)
+
+    tmp_mbox = mkstemp('b4-lookup-mbox')[1]
+    in_mbxf = get_pi_thread_by_url(t_mbx_url, tmp_mbox, nocache=nocache)
     if not in_mbxf:
         return None
     in_mbx = mailbox.mbox(in_mbxf)
