@@ -15,6 +15,7 @@ import re
 import time
 import json
 import fnmatch
+import shutil
 
 import urllib.parse
 import xml.etree.ElementTree
@@ -29,6 +30,8 @@ logger = b4.logger
 def mbox_to_am(mboxfile, cmdargs):
     config = b4.get_main_config()
     outdir = cmdargs.outdir
+    if outdir == '-':
+        cmdargs.nocover = True
     wantver = cmdargs.wantver
     wantname = cmdargs.wantname
     covertrailers = cmdargs.covertrailers
@@ -60,11 +63,16 @@ def mbox_to_am(mboxfile, cmdargs):
         slug = lser.get_slug(extended=True)
         gitbranch = lser.get_slug(extended=False)
 
-    am_filename = os.path.join(outdir, '%s.mbx' % slug)
-    am_cover = os.path.join(outdir, '%s.cover' % slug)
+    if outdir != '-':
+        am_filename = os.path.join(outdir, '%s.mbx' % slug)
+        am_cover = os.path.join(outdir, '%s.cover' % slug)
 
-    if os.path.exists(am_filename):
-        os.unlink(am_filename)
+        if os.path.exists(am_filename):
+            os.unlink(am_filename)
+    else:
+        # Create a temporary file that we will remove later
+        am_filename = mkstemp('b4-am-stdout')[1]
+        am_cover = None
 
     logger.info('---')
     if cmdargs.cherrypick:
@@ -96,6 +104,7 @@ def mbox_to_am(mboxfile, cmdargs):
             cherrypick = list(b4.parse_int_range(cmdargs.cherrypick, upper=len(lser.patches)-1))
     else:
         cherrypick = None
+
     logger.critical('Writing %s', am_filename)
     mbx = mailbox.mbox(am_filename)
     am_mbx = lser.save_am_mbox(mbx, noaddtrailers=cmdargs.noaddtrailers,
@@ -118,9 +127,9 @@ def mbox_to_am(mboxfile, cmdargs):
     if len(lser.trailer_mismatches):
         logger.critical('---')
         logger.critical('NOTE: some trailers ignored due to from/email mismatches:')
-        for tvalue, fname, femail in lser.trailer_mismatches:
-            logger.critical('    ! Trailer: %s', tvalue)
-            logger.critical('         From: %s <%s>', fname, femail)
+        for tname, tvalue, fname, femail in lser.trailer_mismatches:
+            logger.critical('    ! Trailer: %s: %s', tname, tvalue)
+            logger.critical('     Msg From: %s <%s>', fname, femail)
         logger.critical('NOTE: Rerun with -S to apply them anyway')
 
     topdir = None
@@ -144,7 +153,7 @@ def mbox_to_am(mboxfile, cmdargs):
     if not lser.complete:
         logger.critical('WARNING: Thread incomplete!')
 
-    if lser.has_cover:
+    if lser.has_cover and not cmdargs.nocover:
         lser.save_cover(am_cover)
 
     top_msgid = None
@@ -179,7 +188,8 @@ def mbox_to_am(mboxfile, cmdargs):
     if base_commit:
         logger.critical(' Base: %s', base_commit)
         logger.critical('       git checkout -b %s %s', gitbranch, base_commit)
-        logger.critical('       git am %s', am_filename)
+        if cmdargs.outdir != '-':
+            logger.critical('       git am %s', am_filename)
     else:
         cleanmsg = ''
         if topdir is not None:
@@ -215,12 +225,17 @@ def mbox_to_am(mboxfile, cmdargs):
                             cleanmsg = ' (best guess: %s)' % tag
 
         logger.critical(' Base: not found%s', cleanmsg)
-        logger.critical('       git am %s', am_filename)
+        if cmdargs.outdir != '-':
+            logger.critical('       git am %s', am_filename)
 
     am_mbx.close()
-    thanks_record_am(lser, cherrypick=cherrypick)
+    if cmdargs.outdir == '-':
+        logger.info('---')
+        with open(am_filename, 'r') as fh:
+            shutil.copyfileobj(fh, sys.stdout)
+        os.unlink(am_filename)
 
-    return am_filename
+    thanks_record_am(lser, cherrypick=cherrypick)
 
 
 def thanks_record_am(lser, cherrypick=None):
@@ -457,24 +472,14 @@ def main(cmdargs):
         # Force nocache mode
         cmdargs.nocache = True
 
+    savefile = mkstemp('b4-mbox')[1]
+
     if not cmdargs.localmbox:
         msgid = b4.get_msgid(cmdargs)
-        wantname = cmdargs.wantname
-        outdir = cmdargs.outdir
-        if wantname:
-            savefile = os.path.join(outdir, wantname)
-        else:
-            # Save it into msgid.mbox
-            savefile = '%s.t.mbx' % msgid
-            savefile = os.path.join(outdir, savefile)
 
-        mboxfile = b4.get_pi_thread_by_msgid(msgid, savefile, useproject=cmdargs.useproject, nocache=cmdargs.nocache)
-        if mboxfile is None:
+        threadmbox = b4.get_pi_thread_by_msgid(msgid, savefile, useproject=cmdargs.useproject, nocache=cmdargs.nocache)
+        if threadmbox is None:
             return
-
-        # Move it into -thread
-        threadmbox = '%s-thread' % mboxfile
-        os.rename(mboxfile, threadmbox)
     else:
         if os.path.exists(cmdargs.localmbox):
             threadmbox = cmdargs.localmbox
@@ -491,5 +496,21 @@ def main(cmdargs):
             os.unlink(threadmbox)
     else:
         mbx = mailbox.mbox(threadmbox)
-        logger.critical('Saved %s', threadmbox)
         logger.critical('%s messages in the thread', len(mbx))
+        mbx.close()
+        if cmdargs.outdir == '-':
+            logger.info('---')
+            with open(threadmbox, 'r') as fh:
+                shutil.copyfileobj(fh, sys.stdout)
+            os.unlink(threadmbox)
+            return
+
+        if cmdargs.wantname:
+            savefile = os.path.join(cmdargs.outdir, cmdargs.wantname)
+        else:
+            savefile = os.path.join(cmdargs.outdir, '%s.mbx' % msgid)
+
+        shutil.copy(threadmbox, savefile)
+        logger.info('Saved %s', savefile)
+        if not cmdargs.localmbox:
+            os.unlink(threadmbox)
